//-------------------------------------------------------------------
// File    : stm32_ub_uart.h
// Author  : Uwe Becker
// Date    : 2018-01-14
// Version : 0.01
//
// Project : STM32
// Modul   : UART-Library
// UART-Library for STM32F103
//-------------------------------------------------------------------

#ifndef STM32_UB_UART_H_
#define STM32_UB_UART_H_

#include "stm32f1xx.h"
#include "stm32f1xx_hal.h"
#include <stdbool.h>

#include "stm32_ub_gpio.h"
#include "stm32_ub_system.h"

#define UART_PORT_CNT		3		// number of uart ports on this mcu [0..2]


#define UART_BYTE_INTERVALL_ms		1 // delay between bytes in a transfer

// PORT-0 ----------------------------------------------------------------
#define UART0_PORT						USART1
#define UART0_IRQn						USART1_IRQn
#define UART0_IRQ_HANDLER				USART1_IRQHandler

#define UART0_TX_PORT		GPIOB
#define UART0_TX_PIN		GPIO_PIN_6

#define UART0_RX_PORT		GPIOB
#define UART0_RX_PIN		GPIO_PIN_7

// -----------------------------------------------------------------------


// PORT-1 ----------------------------------------------------------------
#define UART1_PORT						USART2
#define UART1_IRQn						USART2_IRQn
#define UART1_IRQ_HANDLER				USART2_IRQHandler

#define UART1_TX_PORT		GPIOA
#define UART1_TX_PIN		GPIO_PIN_2

#define UART1_RX_PORT		GPIOA
#define UART1_RX_PIN		GPIO_PIN_3
// -----------------------------------------------------------------------


// PORT-2 ----------------------------------------------------------------
#define UART2_PORT						USART3
#define UART2_IRQn						USART3_IRQn
#define UART2_IRQ_HANDLER				USART3_IRQHandler

#define UART2_TX_PORT		GPIOB
#define UART2_TX_PIN		GPIO_PIN_10

#define UART2_RX_PORT		GPIOB
#define UART2_RX_PIN		GPIO_PIN_11
// -----------------------------------------------------------------------


#define RX_BUFFER_SIZE                   32	// must be 2^n (128,256,512 ...)



typedef struct {
	UART_HandleTypeDef handle;
	uint32_t rx_flag;
	uint32_t wr_ptr;
	uint32_t rd_ptr;
	uint8_t rx_ringbuffer[RX_BUFFER_SIZE];
	uint8_t rx_buff;
}UART_PORT_t;

typedef struct {
	UART_PORT_t port[UART_PORT_CNT];
	uint32_t rx_mask;
}UART_T;
UART_T UART;


void UB_Uart_Init(uint32_t baudrate, uint8_t mode);
void UB_Uart_Send_Byte(uint8_t port_nr, uint8_t value);
void UB_Uart_Send_Array(uint8_t port_nr, uint8_t *ptr, uint8_t cnt);
void UB_Uart_Flush_Ringbuffer(uint8_t port_nr);
uint32_t UB_Uart_Get_Ringbuffer_Cnt(uint8_t port_nr);
uint8_t UB_Uart_Get_Ringbuffer_Byte(uint8_t port_nr);
bool UB_Uart_Check_Rx_Buf(uint8_t port_nr);
uint8_t UB_Uart_Get_Rx_Buf(uint8_t port_nr);




#endif /* STM32_UB_UART_H_ */
