//-------------------------------------------------------------------
// File    : stm32_ub_system.c
// Author  : Uwe Becker
// Date    : 2018-01-14
// Version : 0.01
//
// Project : STM32
// Modul   : SYSTEM-Library
// System-Library for STM32F103
//-------------------------------------------------------------------

#include "stm32_ub_system.h"

void UB_System_Init(void)
{
	RCC_ClkInitTypeDef clkinitstruct = { 0 };
	RCC_OscInitTypeDef oscinitstruct = { 0 };
	/* Configure PLL ------------------------------------------------------*/
	/* PLL configuration: PLLCLK = (HSI / 2) * PLLMUL = (8 / 2) * 16 = 64 MHz */
	/* PREDIV1 configuration: PREDIV1CLK = PLLCLK / HSEPredivValue = 64 / 1 = 64 MHz */
	/* Enable HSI and activate PLL with HSi_DIV2 as source */
	oscinitstruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	oscinitstruct.HSEState = RCC_HSE_OFF;
	oscinitstruct.LSEState = RCC_LSE_OFF;
	oscinitstruct.HSIState = RCC_HSI_ON;
	oscinitstruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
	oscinitstruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
	oscinitstruct.PLL.PLLState = RCC_PLL_ON;
	oscinitstruct.PLL.PLLSource = RCC_PLLSOURCE_HSI_DIV2;
	oscinitstruct.PLL.PLLMUL = RCC_PLL_MUL16;
	if (HAL_RCC_OscConfig(&oscinitstruct) != HAL_OK) {
		/* Initialization Error */
		while (1)
			;

	}
	/* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2
	 clocks dividers */
	clkinitstruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK
			| RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
	clkinitstruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	clkinitstruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	clkinitstruct.APB2CLKDivider = RCC_HCLK_DIV1;
	clkinitstruct.APB1CLKDivider = RCC_HCLK_DIV2;
	if (HAL_RCC_ClockConfig(&clkinitstruct, FLASH_LATENCY_2) != HAL_OK) {
		/* Initialization Error */
		while (1)
			;

	}
}


void UB_System_GpioClockEnable(GPIO_TypeDef *port)
{
	if(port == GPIOA)  __HAL_RCC_GPIOA_CLK_ENABLE();
	if(port == GPIOB)  __HAL_RCC_GPIOB_CLK_ENABLE();
	if(port == GPIOC)  __HAL_RCC_GPIOC_CLK_ENABLE();
	if(port == GPIOD)  __HAL_RCC_GPIOD_CLK_ENABLE();
	if(port == GPIOE)  __HAL_RCC_GPIOE_CLK_ENABLE();
}

void UB_System_UartClockEnable(USART_TypeDef *port)
{
	if(port == USART1) __HAL_RCC_USART1_CLK_ENABLE();
	if(port == USART2) __HAL_RCC_USART2_CLK_ENABLE();
	if(port == USART3) __HAL_RCC_USART3_CLK_ENABLE();
}

void UB_System_AFIORemapUart(USART_TypeDef *uart_port, GPIO_TypeDef *gpio_port)
{
	if(uart_port == USART1) {
		UB_System_AFIOClockEnable();
		if(gpio_port == GPIOB) {
			__HAL_AFIO_REMAP_USART1_ENABLE();
		}
		else {
			__HAL_AFIO_REMAP_USART1_DISABLE();
		}
		return;
	}
	if(uart_port == USART2) {
		UB_System_AFIOClockEnable();
		if(gpio_port == GPIOD) {
			__HAL_AFIO_REMAP_USART2_ENABLE();
		}
		else {
			__HAL_AFIO_REMAP_USART2_DISABLE();
		}
		return;
	}
	if(uart_port == USART3) {
		UB_System_AFIOClockEnable();
		if(gpio_port == GPIOD) {
			__HAL_AFIO_REMAP_USART3_ENABLE();
		}
		else if(gpio_port == GPIOC) {
			__HAL_AFIO_REMAP_USART3_PARTIAL();
		}
		else {
			__HAL_AFIO_REMAP_USART3_DISABLE();
		}
		return;
	}
}

void UB_System_AFIOClockEnable(void)
{
	__HAL_RCC_AFIO_CLK_ENABLE();
}


void UB_System_Delay_ms(uint32_t delay)
{
	HAL_Delay(delay);
}

uint32_t UB_System_Get_UID(void)
{
	uint32_t value = 0;
	uint32_t buf[3];

	HAL_GetUID(buf);

	value = buf[0];

	return value;
}
