//-------------------------------------------------------------------
// File    : stm32_ub_uart.c
// Author  : Uwe Becker
// Date    : 2018-01-14
// Version : 0.01
//
// Project : STM32
// Modul   : UART-Library
// UART-Library for STM32F103
//-------------------------------------------------------------------


#include "stm32_ub_uart.h"




// init all uart ports
void UB_Uart_Init(uint32_t baudrate, uint8_t mode)
{
	uint32_t n,m;

	UART.rx_mask = 0;

	for(m = 0; m < UART_PORT_CNT; m++) {

	  UART.port[m].wr_ptr = 0;
	  UART.port[m].rd_ptr = 0;

	  for(n=0; n< RX_BUFFER_SIZE; n++) {
		  UART.port[m].rx_ringbuffer[n] = 0x00;
	  }
	  UART.port[m].rx_buff = 0x00;
	}

	// Port-0
#if UART_PORT_CNT >= 1
	UART.port[0].handle.Instance        = UART0_PORT;
	UART.port[0].rx_flag = 0x01;
	UB_System_AFIORemapUart(UART0_PORT, UART0_TX_PORT);
	UB_Gpio_Init_Output_AF(UART0_TX_PORT, UART0_TX_PIN, GPIO_NOPULL, GPIO_SPEED_FREQ_HIGH);
	UB_Gpio_Init_Input_AF(UART0_RX_PORT, UART0_RX_PIN, GPIO_PULLUP, GPIO_SPEED_FREQ_HIGH);
	UB_System_UartClockEnable(UART0_PORT);
#endif

	// Port-1
#if UART_PORT_CNT >= 2
	UART.port[1].handle.Instance        = UART1_PORT;
	UART.port[1].rx_flag = 0x02;
	UB_System_AFIORemapUart(UART1_PORT, UART1_TX_PORT);
	UB_Gpio_Init_Output_AF(UART1_TX_PORT, UART1_TX_PIN, GPIO_NOPULL, GPIO_SPEED_FREQ_HIGH);
	UB_Gpio_Init_Input_AF(UART1_RX_PORT, UART1_RX_PIN, GPIO_PULLUP, GPIO_SPEED_FREQ_HIGH);
	UB_System_UartClockEnable(UART1_PORT);
#endif


	// Port-2
#if UART_PORT_CNT >= 3
	UART.port[2].handle.Instance        = UART2_PORT;
	UART.port[2].rx_flag = 0x04;
	UB_System_AFIORemapUart(UART2_PORT, UART2_TX_PORT);
	UB_Gpio_Init_Output_AF(UART2_TX_PORT, UART2_TX_PIN, GPIO_NOPULL, GPIO_SPEED_FREQ_HIGH);
	UB_Gpio_Init_Input_AF(UART2_RX_PORT, UART2_RX_PIN, GPIO_PULLUP, GPIO_SPEED_FREQ_HIGH);
	UB_System_UartClockEnable(UART2_PORT);
#endif

	for(m = 0; m < UART_PORT_CNT; m++) {
		UART.port[m].handle.Init.BaudRate   = baudrate;
		UART.port[m].handle.Init.WordLength = UART_WORDLENGTH_8B;
		UART.port[m].handle.Init.StopBits   = UART_STOPBITS_1;
		UART.port[m].handle.Init.Parity     = UART_PARITY_NONE;
		UART.port[m].handle.Init.HwFlowCtl  = UART_HWCONTROL_NONE;
		UART.port[m].handle.Init.Mode       = UART_MODE_TX_RX;
		HAL_UART_Init(&UART.port[m].handle);
	}

	// Port-0
#if UART_PORT_CNT >= 1
	HAL_NVIC_SetPriority(UART0_IRQn, 0, 1);
	HAL_NVIC_EnableIRQ(UART0_IRQn);
#endif
	// Port-1
#if UART_PORT_CNT >= 2
	HAL_NVIC_SetPriority(UART1_IRQn, 0, 2);
	HAL_NVIC_EnableIRQ(UART1_IRQn);
#endif
	// Port-2
#if UART_PORT_CNT >= 3
	HAL_NVIC_SetPriority(UART2_IRQn, 0, 3);
	HAL_NVIC_EnableIRQ(UART2_IRQn);
#endif

	  for(m = 0; m < UART_PORT_CNT; m++) {
		  __HAL_UART_ENABLE_IT(&UART.port[m].handle, UART_IT_RXNE);
	  }
}

// send single byte
void UB_Uart_Send_Byte(uint8_t port_nr, uint8_t value)
{
	if(port_nr >= UART_PORT_CNT) return;

	HAL_UART_Transmit(&UART.port[port_nr].handle, (uint8_t *)&value, 1, 1000);
}

// send array of bytes
void UB_Uart_Send_Array(uint8_t port_nr, uint8_t *ptr, uint8_t cnt)
{
	if(port_nr >= UART_PORT_CNT) return;

	HAL_UART_Transmit(&UART.port[port_nr].handle, (uint8_t*)ptr, cnt, 1000);
}

// clear complete ringbuffer
void UB_Uart_Flush_Ringbuffer(uint8_t port_nr)
{
	if(port_nr >= UART_PORT_CNT) return;

	UART.port[port_nr].wr_ptr = 0;
	UART.port[port_nr].rd_ptr = 0;
}


// get number of bytes in rx ringbuffer
uint32_t UB_Uart_Get_Ringbuffer_Cnt(uint8_t port_nr)
{
	uint32_t cnt = 0;

	if(port_nr >= UART_PORT_CNT) return 0;

	if(UART.port[port_nr].wr_ptr >= UART.port[port_nr].rd_ptr) {
		cnt = UART.port[port_nr].wr_ptr - UART.port[port_nr].rd_ptr;
	}
	else {
		cnt = (UART.port[port_nr].wr_ptr + RX_BUFFER_SIZE) - UART.port[port_nr].rd_ptr;
	}

	return (cnt);
}

// read single byte from ringbuffer
uint8_t UB_Uart_Get_Ringbuffer_Byte(uint8_t port_nr)
{
	uint8_t value = 0;

	if(port_nr >= UART_PORT_CNT) return 0;

	value = UART.port[port_nr].rx_ringbuffer[UART.port[port_nr].rd_ptr];
	UART.port[port_nr].rd_ptr++;
	UART.port[port_nr].rd_ptr &= (RX_BUFFER_SIZE-1);

	return (value);
}

// true, if rx_buff has received a byte
bool UB_Uart_Check_Rx_Buf(uint8_t port_nr)
{
	if(port_nr >= UART_PORT_CNT) return false;

	if((UART.rx_mask & UART.port[port_nr].rx_flag) != 0) return true;

	return false;
}

// read byte from rx_buff
uint8_t UB_Uart_Get_Rx_Buf(uint8_t port_nr)
{
	uint8_t value = 0;

	if(port_nr >= UART_PORT_CNT) return false;

	value = UART.port[port_nr].rx_buff;
	UART.rx_mask &= ~UART.port[port_nr].rx_flag; // clear flag

	return (value);
}


#if UART_PORT_CNT >= 1
void UART0_IRQ_HANDLER(void)
{
	volatile uint8_t value;

	if(__HAL_UART_GET_IT_SOURCE(&UART.port[0].handle, UART_IT_RXNE))
	{
		value=(uint8_t)(UART0_PORT->DR & 0xFF);
		UART.port[0].rx_ringbuffer[UART.port[0].wr_ptr] = value;
		UART.port[0].wr_ptr++;
		UART.port[0].wr_ptr &= (RX_BUFFER_SIZE-1);

		UART.port[0].rx_buff = value;
		UART.rx_mask |= UART.port[0].rx_flag; // set flag
	}
}
#endif

#if UART_PORT_CNT >= 2
void UART1_IRQ_HANDLER(void)
{
	volatile uint8_t value;

	if(__HAL_UART_GET_IT_SOURCE(&UART.port[1].handle, UART_IT_RXNE))
	{
		value=(uint8_t)(UART1_PORT->DR & 0xFF);
		UART.port[1].rx_ringbuffer[UART.port[1].wr_ptr] = value;
		UART.port[1].wr_ptr++;
		UART.port[1].wr_ptr &= (RX_BUFFER_SIZE-1);

		UART.port[1].rx_buff = value;
		UART.rx_mask |= UART.port[1].rx_flag; // set flag
	}
}
#endif


#if UART_PORT_CNT >= 3
void UART2_IRQ_HANDLER(void)
{
	volatile uint8_t value;

	if(__HAL_UART_GET_IT_SOURCE(&UART.port[2].handle, UART_IT_RXNE))
	{
		value=(uint8_t)(UART2_PORT->DR & 0xFF);
		UART.port[2].rx_ringbuffer[UART.port[2].wr_ptr] = value;
		UART.port[2].wr_ptr++;
		UART.port[2].wr_ptr &= (RX_BUFFER_SIZE-1);

		UART.port[2].rx_buff = value;
		UART.rx_mask |= UART.port[2].rx_flag; // set flag
	}
}
#endif




