//-------------------------------------------------------------------
// File    : ub_xnet_l0.c
// Author  : Uwe Becker
// Date    : 2018-01-14
// Version : 0.01
//
// Project : XNET
// Modul   : Protocol layer L0 [hardware layer]
// MCU and hardware related functions
//-------------------------------------------------------------------


// include own header file
#include "ub_xnet_l0.h"
#include "stm32_ub_uart.h"  // stm32f1 uart library
#include "stm32_ub_gpio.h"   // stm32f1 gpio library




// init xnet layer-0
// this function must initialize all uart ports on this node
// and if necessary a GPIO for a status LED
void UB_Xnet_L0_Init(void)
{
	// init UART library
	UB_Uart_Init(XNET_UART_BAUDRATE, XNET_UART_MODE);

	// init gpio
	UB_Gpio_Init_Output(GPIOC, GPIO_PIN_13, GPIO_NOPULL, GPIO_SPEED_FREQ_LOW);
}

// delay function
// this function must realize a blocking delay in ms
void UB_Xnet_L0_Delay_ms(U32_DWORD delay_ms)
{
	UB_System_Delay_ms(delay_ms);
}

// check rx buffer
// this function must return XTRUE, if the uart port has received anything
U8_BOOL UB_Xnet_L0_Check_Rx_Buf(U8_CHAR uart_port)
{
	if(UB_Uart_Check_Rx_Buf(uart_port) == true) {
		return XTRUE;
	}
	else {
		return XFALSE;
	}
}

// read byte
// this function must readout a single received byte from the uart port
U8_CHAR UB_Xnet_L0_Read_Rx_Buf(U8_CHAR uart_port)
{
	U8_CHAR rx_value;

	rx_value = UB_Uart_Get_Rx_Buf(uart_port);

	return rx_value;
}

// send byte
// this function must send a single byte to an uart port
void UB_Xnet_L0_Send_Byte(U8_CHAR uart_port, U8_CHAR tx_value)
{
	UB_Uart_Send_Byte(uart_port, tx_value);
}

// send array
// this function must send an array of bytes to an uart port
void UB_Xnet_L0_Send_Array(U8_CHAR uart_port, U8_CHAR *ptr, U8_CHAR size)
{
	UB_Uart_Send_Array(uart_port, ptr, size);
}


// clear rx ringbufffer
// this function must clear the RX ringbuffer
void UB_Xnet_L0_Clear_Rx_Ringbuffer(U8_CHAR uart_port)
{
	UB_Uart_Flush_Ringbuffer(uart_port);
}

// get rx ringbuffer counts
// this function must get the number of received bytes in the rx ringbuffer
U8_CHAR UB_Xnet_L0_Get_Rx_Ringbuffer_Cnt(U8_CHAR uart_port)
{
	U8_CHAR nr_bytes;

	nr_bytes = UB_Uart_Get_Ringbuffer_Cnt(uart_port);

	return nr_bytes;
}

// readout rx ringbuffer
// this function must readout the first received byte from the uart port in the rx ringbuffer
U8_CHAR UB_Xnet_L0_Read_Rx_Ringbuffer(U8_CHAR uart_port)
{
	U8_CHAR rx_value;

	rx_value = UB_Uart_Get_Ringbuffer_Byte(uart_port);

	return rx_value;
}

// this function must turn a status LED on
void UB_Xnet_L0_Status_Led_On(void)
{
	UB_Gpio_Write(GPIOC, GPIO_PIN_13, 0);
}

// this function must turn a status LED off
void UB_Xnet_L0_Status_Led_Off(void)
{
	UB_Gpio_Write(GPIOC, GPIO_PIN_13, 1);
}
